package com.tzld.piaoquan.content.understanding.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.tzld.piaoquan.content.understanding.common.enums.ExceptionEnum;
import com.tzld.piaoquan.content.understanding.common.exception.CommonException;
import com.tzld.piaoquan.content.understanding.dao.mapper.CuPipelineMapper;
import com.tzld.piaoquan.content.understanding.dao.mapper.CuPipelineStepMapper;
import com.tzld.piaoquan.content.understanding.dao.mapper.CuPromptMapper;
import com.tzld.piaoquan.content.understanding.dao.mapper.CuTaskMapper;
import com.tzld.piaoquan.content.understanding.model.dto.ContentAnalyseDTO;
import com.tzld.piaoquan.content.understanding.model.dto.ContentDTO;
import com.tzld.piaoquan.content.understanding.model.dto.TreeNode;
import com.tzld.piaoquan.content.understanding.model.param.ActionParam;
import com.tzld.piaoquan.content.understanding.model.param.ContentAnalyseParam;
import com.tzld.piaoquan.content.understanding.model.po.CuPipelineStep;
import com.tzld.piaoquan.content.understanding.model.po.CuPipelineStepExample;
import com.tzld.piaoquan.content.understanding.model.po.CuPrompt;
import com.tzld.piaoquan.content.understanding.model.po.CuTask;
import com.tzld.piaoquan.content.understanding.service.Action;
import com.tzld.piaoquan.content.understanding.service.PipelineService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author supeng
 */
@Slf4j
@Service
public class PipelineServiceImpl implements PipelineService {

    @Autowired
    private CuPipelineMapper cuPipelineMapper;

    @Autowired
    private CuPipelineStepMapper cuPipelineStepMapper;

    @Autowired
    private CuPromptMapper cuPromptMapper;

    @Autowired
    private Map<String, Action> actionMap;

    @Autowired
    private MongoTemplate mongoTemplate;

    private static final String COLLECTION_NAME = "content_understanding_info";

    @Autowired
    private CuTaskMapper cuTaskMapper;

    @Override
    public void execute(ContentAnalyseParam param) {
        // 1.获取pipeline配置
        Long pipelineId = param.getPipelineId();
        String content = param.getContent();
        Integer type = param.getType();
        CuPipelineStepExample example = new CuPipelineStepExample();
        example.createCriteria().andPipelineIdEqualTo(pipelineId);
        List<CuPipelineStep> stepList = cuPipelineStepMapper.selectByExample(example);
        if (Objects.isNull(stepList) || stepList.isEmpty()) {
            throw new CommonException(ExceptionEnum.CONFIG_ERROR, "无配置 pipelineId：" + pipelineId);
        }
        // 2.构建tree结构
        TreeNode root = buildTree(stepList);
        root.setType(type);
        root.setInput(content);
        ContentAnalyseDTO dto = new ContentAnalyseDTO();
        BeanUtils.copyProperties(param, dto);
        // 3.按照步骤执行pipeline每一步动作
        executeTreeNodeBFS(root, dto);
    }

//    @Override
//    public void execute(ContentUnderstandDTO dto) {
//        // 1.数据准备
//        Long pipelineId = dto.getPipelineId();
//        // 1.新增任务
//        String taskId = UUID.randomUUID().toString().replace("-", "") + System.currentTimeMillis();
//        CuTask cuTask = new CuTask();
//        cuTask.setTaskId(taskId);
//        cuTask.setPipelineId(pipelineId);
////        cuTask.setInput();
//        int insertCount = cuTaskMapper.insertSelective(cuTask);
//        // 2.获取pipeline配置
//        Integer contentType = dto.getContentType();
//        CuPipelineStepExample example = new CuPipelineStepExample();
//        example.createCriteria().andPipelineIdEqualTo(pipelineId);
//        List<CuPipelineStep> stepList = cuPipelineStepMapper.selectByExample(example);
//        if (Objects.isNull(stepList) || stepList.isEmpty()) {
//            throw new CommonException(ExceptionEnum.CONFIG_ERROR, "无配置 pipelineId：" + pipelineId);
//        }
//        // 3.构建tree结构
//        TreeNode root = buildTree(stepList);
//        root.setType(contentType);
//        String content = getContent(dto, contentType);
//        if (Objects.isNull(content)) {
//            throw new CommonException(ExceptionEnum.DATA_ERROR, "数据异常content：" + content);
//        }
//        root.setInput(content);
//        ContentAnalyseDTO dto1 = new ContentAnalyseDTO();
//        dto1.setVideoId(dto.getVideoId());
//        dto1.setContent(content);
//        // 4.按照步骤执行pipeline每一步动作
//        executeTreeNodeBFS(root, dto1);
//    }

    @Override
    public void execute(ContentDTO contentDTO) {
        // 1.数据准备
        Long pipelineId = contentDTO.getPipelineId();
        String content = contentDTO.getContent();
        // 1.新增任务
        String taskId = UUID.randomUUID().toString().replace("-", "") + System.currentTimeMillis();
        CuTask cuTask = new CuTask();
        cuTask.setTaskId(taskId);
        cuTask.setPipelineId(pipelineId);
        cuTask.setInput(content);
        int insertCount = cuTaskMapper.insertSelective(cuTask);
        String reason = "";
        if (insertCount <= 0) {
            throw new CommonException(ExceptionEnum.SYSTEM_ERROR, "");
        }
        // 2.获取pipeline配置
        Integer contentType = contentDTO.getContentType();
        CuPipelineStepExample example = new CuPipelineStepExample();
        example.createCriteria().andPipelineIdEqualTo(pipelineId);
        List<CuPipelineStep> stepList = cuPipelineStepMapper.selectByExample(example);
        if (Objects.isNull(stepList) || stepList.isEmpty()) {
            throw new CommonException(ExceptionEnum.CONFIG_ERROR, "无配置 pipelineId：" + pipelineId);
        }
        // 3.构建tree结构
        TreeNode root = buildTree(stepList);
        root.setType(contentType);
        if (Objects.isNull(content)) {
            throw new CommonException(ExceptionEnum.DATA_ERROR, "数据异常content：" + content);
        }
        root.setInput(content);
        ContentAnalyseDTO dto1 = new ContentAnalyseDTO();
        dto1.setVideoId(contentDTO.getVideoId());
        dto1.setContent(content);
        // 4.按照步骤执行pipeline每一步动作
        executeTreeNodeBFS(root, dto1);
    }

//    private String getContent(ContentUnderstandDTO dto, Integer contentType) {
//        String content = null;
//        if (Objects.equals(contentType, ContentTypeEnum.TITLE.getValue())) {
//            content = dto.getTitle();
//        } else if (Objects.equals(contentType, ContentTypeEnum.COVER.getValue())) {
//            content = dto.getCoverUrl();
//        } else if (Objects.equals(contentType, ContentTypeEnum.VIDEO.getValue())) {
//            content = dto.getVideoUrl();
//        } else if (Objects.equals(contentType, ContentTypeEnum.INTRODUCTION.getValue())) {
//            content = dto.getIntroduction();
//        } else if (Objects.equals(contentType, ContentTypeEnum.AUDIO.getValue())) {
//            content = dto.getAudioUrl();
//        } else if (Objects.equals(contentType, ContentTypeEnum.SRT_VTT.getValue())) {
//            content = dto.getSrt();
//        }
//        return content;
//    }

    public TreeNode buildTree(List<CuPipelineStep> stepList) {
        Map<Long, TreeNode> nodeMap = new HashMap<>();
        TreeNode root = null;

        // 将所有节点放入 Map 中，方便后续查找
        for (CuPipelineStep step : stepList) {
            TreeNode node = new TreeNode();
            node.setStep(step);
            nodeMap.put(step.getId(), node);
        }

        // 构建父子关系
        for (CuPipelineStep step : stepList) {
            Long parentId = step.getParentId();
            if (Objects.isNull(parentId) || parentId == 0) {
                root = nodeMap.get(step.getId());
            } else if (nodeMap.containsKey(parentId)) {
                TreeNode parentNode = nodeMap.get(parentId);
                TreeNode currentNode = nodeMap.get(step.getId());
                List<TreeNode> children = parentNode.getChildren();
                if (Objects.isNull(children)) {
                    children = new ArrayList<>();
                    parentNode.setChildren(children);
                }
                children.add(currentNode);
            }
        }
        return root;
    }

    /**
     * 遍历执行
     *
     * @param root 根节点
     * @param dto  数据
     */
    public void executeTreeNodeBFS(TreeNode root, ContentAnalyseDTO dto) {
        if (Objects.isNull(root) || Objects.isNull(dto) || Objects.isNull(dto.getVideoId())) {
            return;
        }
        Queue<TreeNode> queue = new LinkedList<>();
        queue.offer(root);

        Long videoId = dto.getVideoId();

        while (!queue.isEmpty()) {
            TreeNode currentNode = queue.poll();
            CuPipelineStep step = currentNode.getStep();
            Integer type = currentNode.getType();
            String input = currentNode.getInput() == null ? "" : currentNode.getInput();
            // 执行action
            String result = executeStep(step, input, type);
            if (Objects.isNull(currentNode.getChildren()) || currentNode.getChildren().isEmpty()) {
                if (Objects.nonNull(result)) {
                    //异常不阻塞其他分支执行
                    try {
                        Map<String, Object> resultMap = JSON.parseObject(result, new TypeReference<Map<String, Object>>() {
                        });
                        resultMap.put("video_id", videoId);
                        //保存数据到mongo
                        mongoTemplate.insert(resultMap, COLLECTION_NAME);
                        //保存数据到mysql

                    } catch (Exception e) {
                        log.error("step execute error step = {}, result = {}", e, JSON.toJSONString(step), result);
                    }
                }
                continue;
            }
            for (TreeNode child : currentNode.getChildren()) {
                child.setInput(result);
                queue.offer(child);
            }
        }
    }

    /**
     * 执行step 动作
     *
     * @param step
     * @param input
     * @return
     */
    public String executeStep(CuPipelineStep step, String input, Integer type) {
        CuPrompt cuPrompt = cuPromptMapper.selectByPrimaryKey(step.getPromptId());
        if (Objects.isNull(cuPrompt) || Objects.isNull(cuPrompt.getId())) {
            return null;
        }
        String prompt = String.format(cuPrompt.getPrompt(), input);

        Action action = actionMap.get(step.getAction());
        ActionParam actionParam = new ActionParam();
        actionParam.setType(type);
        actionParam.setInput(input);
        actionParam.setPrompt(prompt);
        return action.execute(actionParam);
    }
}
