package com.tzld.piaoquan.content.understanding.util;

import java.util.function.LongSupplier;
import java.util.function.Predicate;
import java.util.function.Supplier;

public class RetryUtil {

    /**
     * 重试执行操作，直到成功或达到最大重试次数。
     *
     * @param operation      需要执行的操作，返回操作结果。
     * @param successPredicate 判断操作结果是否成功的断言。
     * @param maxRetries     最大重试次数。
     * @param operationName  操作名称，用于日志记录。
     * @param <T>            操作结果的类型。
     * @return 操作成功的结果，如果所有重试都失败则返回 null。
     */
    public static <T> T retry(Supplier<T> operation, Predicate<T> successPredicate, int maxRetries, String operationName) {
        T result = null;
        int attempts = 0;

        while (attempts <= maxRetries) {
            attempts++;
            try {
                result = operation.get();
                if (successPredicate.test(result)) {
                    return result;
                } else {
                    if (attempts <= maxRetries) {
                        // 可以添加适当的重试间隔，例如 Thread.sleep(1000);
                    }
                }
            } catch (Exception e) {
                if (attempts <= maxRetries) {
                    // 可以添加适当的重试间隔，例如 Thread.sleep(1000);
                }
            }
        }

        return null; // 返回 null 表示所有重试都失败
    }
    public static void main(String[] args) {
        // 封装带重试的插入操作
        RetryUtil.retry(
                () -> 1,
                insertCount -> insertCount > 0, // 判断插入影响行数是否大于 0
                3,
                "插入 CuTask"
        );
    }
}
